#' Correction Workflow for Hand-Corrected Written Exams
#' 
#' This function will do the whole correction workflow for hand-corrected exams: 
#' Read in (or create) points file, apply the grading scale, export the grades in 
#' edoz format and knit the corresponding information into a tex-file (used later 
#' on for the overview PDF).
#'
#' @param manual.pointsFile File name containing points. If not existing, an
#' appropriate file will be created (with commas as separators).
#' @param manual.pointsFile.sep Separator used in \code{manual.pointsFile}.
#' @param edozFile File name of edoz file.
#' @param semesterPAfile File name of semester performance file.
#' @param gradeFunction.name Name of the grading function, e.g., \code{"gradesPiecewiseLinear"}.
#' @param gradeFunction.settings A list containting the options for \code{gradeFunction.name}.
#' @param rdataFile If not \code{NULL}, the relevant R objects will be stored in this file.
#' @param minPoints Lower end when \emph{printing} grading scale.
#' @param maxPoints Upper end when \emph{printing} grading scale.
#' @param maxPoints.q Vector containing the maximal points for each question 
#' (only relevant for the plots for each question).
#' @param stepsize Stepsize used when \emph{printing} grading scale.
#'
#' @return knitted results
#' @export
process_corrections_manual <- function(manual.pointsFile,
                                       manual.pointsFile.sep,
                                       edozFile,
                                       semesterPAfile=NULL,
                                       gradeFunction.name,
                                       gradeFunction.settings,
                                       rdataFile,
                                       # from here: only needed in Rnw chunks
                                       minPoints,
                                       maxPoints,
                                       maxPoints.q,
                                       stepsize) {
  
  
  #######################
  ## Read in all files ##
  #######################
  
  ## Read data from edoz exported file
  edozList <- readEdoz(file = edozFile)
  
  if(file.exists(manual.pointsFile)){
    message("Using existing manual grading file ", manual.pointsFile)
  }else{
    createManual(output.file = manual.pointsFile, edoz.file = edozFile, 
                 n.questions = length(maxPoints.q))
    stop("--> Created manual grading file, please fill in and run again! <--")
  }
  
  ## Read manually entered points
  manual.points <- readManual(file = manual.pointsFile, sep = manual.pointsFile.sep)
  
  ## Read file with grade bonus (semester performance)
  if(!is.null(semesterPAfile))
    semesterPA <- readSemesterPA(file = semesterPAfile)
  
  #########################
  ## Apply grading scale ##
  #########################
  
  ## Transform points into grade by applying grading function
  grade.matrix <- applyGrading(pointsTotal = manual.points$pointsTotal, 
                               gradeFn     = get(gradeFunction.name),
                               settings    = gradeFunction.settings)
  
  
  ## Append raw grades to manual.points, potentially merge with semester performance
  if(!is.null(semesterPAfile)){
    manual.points[,"rawGradeExam"] <- grade.matrix[,"rawGrade"]
    manual.points <- merge(x = manual.points, y = semesterPA, by = "number", all.x = TRUE)
    manual.points[is.na(manual.points[,"gradeBonus"]), "gradeBonus"] <- 0
    manual.points[,"rawGrade"] <- manual.points[,"rawGradeExam"] + manual.points[,"gradeBonus"]
    manual.points[,"grade"]    <- gradeRound(manual.points[,"rawGrade"])
  }else{
    manual.points[,"rawGrade"] <- grade.matrix[,"rawGrade"]
    manual.points[,"grade"]    <- grade.matrix[,"grade"]
  }
  manual.points[,"roundEffect"] <- manual.points[,"grade"] - manual.points[, "rawGrade"]
  
  
  ###############################
  ## Export grades to edozFile ##
  ###############################
  
  edoz.out <- exportGrades(edozList = edozList, results = manual.points, 
                           file = edozFile)
  
  ## Store and export grades to rda-file (if requested)
  if(!is.null(rdataFile))
    save(manual.points, 
         gradeFunction.name,
         gradeFunction.settings, 
         file = rdataFile)
  
  ############################
  ## Knit the overview file ##
  ############################
  
  dir.create("childs", showWarnings = FALSE) ## TODO: More elegant solution???
  
  in_file <- system.file("Rnw-files", "output-file-manual.Rnw", package = "sfsGrading")
  text_input <- paste0(readLines(in_file), collapse = "\n")
  
  knit(text = text_input, output = "childs/output-file-manual.tex", envir = environment())
}




